//===----------- CriticalEdgeRemoval_Fer - Break critical edges ----------===//
//
//                     The LLVM Compiler Infrastructure
//
// This files was originally developed by the LLVM research group. Fernando
// at cs dot ucla dot edu has changed the file a little, so it could be used as
// a pre-register allocation pass.
//
//===---------------------------------------------------------------------===//
//
// CriticalEdgeRemoval pass - Break all of the critical edges in the CFG by
// inserting a dummy basic block.  This pass may be "required" by passes that
// cannot deal with critical edges. Notice that this pass invalidates the
// CFG, because the same BasicBlock is used as parameter for the src
// MachineBasicBlock and the new dummy MachineBasicBlock.
//
//===---------------------------------------------------------------------===//


#ifndef CRITICAL_EDGE_REMOVAL_H
#define CRITICAL_EDGE_REMOVAL_H

#include "llvm/Pass.h"

namespace llvm {

    class CriticalEdgeRemoval_Fer : public MachineFunctionPass {

        public:
 

        /// I am not sure that this pass conserves all these properties, but I
        /// am  assuming that critical edges between BasicBlocks have already
        /// been broken before this pass is executed. So, I only have to break
        /// critical edges between basic blocks split due to new optimizations.
        /// If the underline BasicBlock is the same for both source and target
        /// of the critical edge, this pass will not invalidate the other
        /// passes.
        virtual void getAnalysisUsage(AnalysisUsage & AU) const;

  
        /// This method split all the critical edges in the machine function
        /// control flow graph.
        virtual bool runOnMachineFunction(MachineFunction &mf);
 

        ///
        /// This method breaks the critical edge that links src to dst. It will
        /// insert a dummy block in between these two machine blocks. If they
        /// do not share the same underling BasicBlock, this method will report
        /// an error. Therefore, it is the responsability of the client to
        /// ensure that the control flow graph of BasicBlocks does not contain
        /// critical edges. To ensure this, call
        /// PM.add(createBreakCriticalEdgesPass()); in the target machine.
        void split_critical_edge
         (MachineBasicBlock& src, MachineBasicBlock& dst, MachineFunction& mf);
 
        private:

    };

}

#endif
